"""
Experimental features package for Claude MPM.

This package contains experimental features and enhancements that are
under development or testing. These features may be unstable or subject
to change in future versions.

This __init__.py file is required to make this directory recognizable as a
Python package by setuptools for proper distribution and imports.
"""
